import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:intl_phone_number_input/intl_phone_number_input.dart';
import 'package:pin_input_text_field/pin_input_text_field.dart';
import 'package:slide_countdown_clock/slide_countdown_clock.dart';
import 'package:firebase_phone_verify_ui/bloc/app_bloc.dart';

import 'enum.dart';
import 'enum.dart';

enum OtpStatus {
  sendingSMS,
  phoneInvalide,
  couldNotSentCode,
  smsSentCompleted,
  showOtpBox,
  codeVerifiedCompleted,
  verifying,
  invalideOTP,
  verifyCodeFailed,
  error,
}

class OTPScreen extends StatefulWidget {
  final PhoneNumber phoneNumber;
  OTPScreen(this.phoneNumber);

  _OTPScreenState createState() => _OTPScreenState();
}

class _OTPScreenState extends State<OTPScreen> {
  String uuid;
  AppBloc _bloc = AppBloc();
  String phoneIsoCode = "+855";
  String _verificationId;
  bool _allowResent = false;

  final _editController = TextEditingController();
  final FirebaseAuth _auth = FirebaseAuth.instance;

  @override
  void initState() {
    super.initState();

    Future.delayed(
      Duration(milliseconds: 500),
      () {
        _sendSMSCode();
        //_loginWithPhone("+85593401361");
      },
    );
  }

  @override
  void setState(VoidCallback fn) {
    if (mounted) {
      super.setState(fn);
    }
  }

  _verifyCode() {
    _bloc.add(AppChangeEvent(OtpStatus.verifying));

    var credential = PhoneAuthProvider.credential(
        verificationId: _verificationId, smsCode: _editController.text);
    _auth.signInWithCredential(credential).then((result) {
      _bloc.add(AppChangeEvent(OtpStatus.codeVerifiedCompleted));
      print("_______signInWithCredential___________success_________$result");
      Navigator.pop(context, FirebaseVerifyResult.verifySuccess);
    }).catchError((onError) {
      _bloc.add(AppChangeEvent(OtpStatus.invalideOTP));
      print("_______signInWithCredential___________error_________$onError");
    });
  }

  _codeSent(String verificationId, [int n]) {
    _verificationId = verificationId;

    _bloc.add(AppChangeEvent(OtpStatus.smsSentCompleted));
    print("____code_sent_completd");

    Future.delayed(Duration(milliseconds: 200), () {
      _bloc.add(AppChangeEvent(OtpStatus.showOtpBox));
    });
  }

  _verificationCompleted(AuthCredential authCredential) {
    print("_veryfyCodeCompleted__$authCredential");
    _bloc.add(AppChangeEvent(OtpStatus.codeVerifiedCompleted));
  }

  _codeAutoRetrievalTimeout(String s) {
    print("__codeAutoRetrievalTimeout____$s");
  }

  _verificationFailed(FirebaseAuthException f) {
    _bloc.add(AppChangeEvent(OtpStatus.verifyCodeFailed));

    print("_verificationFailed_______${f.code}________${f.message}");
  }

  _sendSMSCode() async {
    _bloc.add(AppChangeEvent(OtpStatus.sendingSMS));

    String phone = widget.phoneNumber.phoneNumber;
    _auth.verifyPhoneNumber(
      phoneNumber: phone,
      timeout: Duration(minutes: 2),
      codeSent: _codeSent,
      verificationCompleted: _verificationCompleted,
      codeAutoRetrievalTimeout: _codeAutoRetrievalTimeout,
      verificationFailed: _verificationFailed,
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        resizeToAvoidBottomPadding: false,
        appBar: AppBar(
          title: Text("OTP"),
          centerTitle: true,
        ),
        body: BlocProvider(
          create: (context) => _bloc,
          child: BlocConsumer<AppBloc, AppState>(
            listener: (context, snapshot) {},
            builder: (context, state) {
              print(state.status.toString());
              return _buildScreen(state);
            },
          ),
        ));
  }

  Stack _buildScreen(AppState state) {
    return Stack(
      children: <Widget>[
        Padding(
          padding: const EdgeInsets.only(top: 32, left: 16, right: 16),
          child: Container(
              width: double.infinity,
              child: Builder(
                builder: (context) {
                  switch (state.status) {
                    case OtpStatus.sendingSMS:
                      return _buildSendingCode();
                      break;

                    case OtpStatus.smsSentCompleted:
                      return _buildSmsSendCompleted();
                      break;

                    case OtpStatus.verifying:
                      return _buildVerifying();
                      break;

                    case OtpStatus.invalideOTP:
                      return _buildInvalidCode();
                      break;

                    case OtpStatus.codeVerifiedCompleted:
                      return _buildVerifyCompleted();
                      break;

                    case OtpStatus.phoneInvalide:
                      return _buildPhoneInvalid(context);
                      break;
                    case OtpStatus.showOtpBox:
                      return _buildOTPScreen();

                    case OtpStatus.error:
                      return _buildError(context);
                      break;

                    default:
                      return Container();
                      break;
                  }
                },
              )),
        ),
      ],
    );
  }

  Column _buildSmsSendCompleted() {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        Text("Code was sent.", style: TextStyle(fontSize: 17)),
        SizedBox(height: 16),
        Container(
          width: 100,
          height: 100,
          child: Icon(Icons.check_circle_outline, size: 100),
        ),
      ],
    );
  }

  Column _buildVerifying() {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        Text("Verifying!", style: TextStyle(fontSize: 17)),
        SizedBox(height: 16),
        Container(
          width: 70,
          height: 70,
          child: CircularProgressIndicator(),
        ),
      ],
    );
  }

  Column _buildSendingCode() {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        Text("Sending SMS with your code.", style: TextStyle(fontSize: 17)),
        SizedBox(height: 16),
        Container(
          width: 70,
          height: 70,
          child: CircularProgressIndicator(),
        ),
      ],
    );
  }

  Column _buildInvalidCode() {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        Text("Invalide code!", style: TextStyle(fontSize: 17)),
        SizedBox(height: 24),
        Container(
          width: 100,
          height: 100,
          child: Icon(Icons.error_outline, size: 100),
        ),
        SizedBox(height: 24),
        Container(
          width: double.infinity,
          color: Colors.red,
          child: FlatButton(
            child: Text("Try again", style: TextStyle(color: Colors.white)),
            onPressed: () {
              _sendSMSCode();
              _bloc.add(AppChangeEvent(OtpStatus.sendingSMS));
            },
          ),
        ),
      ],
    );
  }

  Column _buildVerifyCompleted() {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        Text("Phone verified.", style: TextStyle(fontSize: 17)),
        SizedBox(height: 16),
        Container(
          width: 100,
          height: 100,
          child: Icon(Icons.check_circle_outline, size: 100),
        ),
      ],
    );
  }

  Column _buildPhoneInvalid(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        Text("Please enter a valid phone number.",
            style: TextStyle(fontSize: 17)),
        SizedBox(height: 24),
        Container(
          width: 100,
          height: 100,
          child: Icon(Icons.error_outline, size: 100),
        ),
        SizedBox(height: 24),
        Container(
          width: double.infinity,
          color: Colors.red,
          child: FlatButton(
            child: Text("Try again", style: TextStyle(color: Colors.white)),
            onPressed: () {
              Navigator.pop(context);
            },
          ),
        ),
      ],
    );
  }

  Column _buildError(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        Text("Something went wrong.", style: TextStyle(fontSize: 17)),
        SizedBox(height: 24),
        Container(
          width: 100,
          height: 100,
          child: Icon(Icons.error_outline, size: 100),
        ),
        SizedBox(height: 24),
        FlatButton(
          child: Text("Try again"),
          onPressed: () {
            Navigator.pop(context);
          },
        ),
      ],
    );
  }

  Stack _buildOTPScreen() {
    return Stack(
      children: <Widget>[
        Padding(
          padding: const EdgeInsets.only(top: 16, left: 32, right: 32),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              Text("Verification code", style: TextStyle(fontSize: 27)),
              SizedBox(height: 24),
              Text("Please enter the OTP sent on your registered phone number.",
                  textAlign: TextAlign.center),
              SizedBox(height: 16),
              PinInputTextField(
                controller: _editController,
                pinLength: 6,
                decoration: BoxLooseDecoration(
                  strokeColor: Theme.of(context).primaryColor,
                  hintText: "------",
                ),
                focusNode: FocusNode(),
                onChanged: (t) {
                  print(t);
                  setState(() {});
                },
              ),
              SizedBox(height: 32),
              !_allowResent
                  ? SlideCountdownClock(
                      duration: Duration(minutes: 2),
                      slideDirection: SlideDirection.Down,
                      separator: ":",
                      shouldShowDays: false,
                      textStyle: TextStyle(
                        color: Colors.black38,
                        fontSize: 27,
                        fontWeight: FontWeight.normal,
                      ),
                      onDone: () {
                        setState(() {
                          _allowResent = true;
                        });
                      },
                    )
                  : Row(
                      mainAxisAlignment: MainAxisAlignment.center,
                      mainAxisSize: MainAxisSize.min,
                      children: <Widget>[
                        Text("Didn't you received any code?",
                            style: TextStyle(fontSize: 12)),
                        FlatButton(
                          child: Text("Resent OTP"),
                          onPressed: () {
                            _sendSMSCode();
                            _bloc.add(AppChangeEvent(OtpStatus.sendingSMS));
                            _allowResent = false;
                          },
                        )
                      ],
                    ),
              SizedBox(height: 24),
              Container(
                width: double.infinity,
                height: 44,
                child: RaisedButton(
                  child:
                      Text("Submit", style: Theme.of(context).textTheme.button),
                  onPressed:
                      _editController.text.length == 6 ? _verifyCode : null,
                ),
              )
            ],
          ),
        ),
      ],
    );
  }
}
